#ifndef SPARSE_H
#define SPARSE_H
// $Id: sparse.h,v 1.1 2001/08/14 08:43:40 dvermeir Exp $

#include <map>

// This class uses maps to implement 2-dimensional sparse matrices.
// Key1 and Key2 are the index types. The default value is an argument
// of the constructor.
template<typename Key1, typename Key2, typename Value>
class SparseMatrix {
public:
  class Element; // forward declaration

  //////////////////////////////////////////////////////////////////////////////////////////
  // Nested class SparseMatrix::Row which refers to a row in the matrix.
  class Row { // a row is a [Key2->Value] map
  friend class Element;
  public:
   Row(SparseMatrix& matrix): matrix_(matrix) {} // Remember matrix for getting default value.
   Element operator[](const Key2& k2) { return Element(*this,k2); }
  private:
   Row& operator=(const Row&); // Forbid assignment
   SparseMatrix& 	matrix_;
   map<Key2,Value>	columns_; // one element for every non-default element in row
  };

  //////////////////////////////////////////////////////////////////////////////////////////
  // Nested class SparseMatrix::Element which refers to an element in a row.
  class Element {
  public:
    Element(Row& r,Key2 k2): row_(r), key2_(k2) {}
    void operator=(const Value& val) {  // E.g. a[k1][k2] = val.
      if (val==matrix().default_value())  // Val is default, erase possible k2 entry from row
	row_.columns_.erase(key2_); 
      else
	row_.columns_[key2_] = val;  // Non-default val: store in row[k2].
      }
    operator Value() { // Return value of a[k1][k2].
      if (row_.columns_.count(key2_)>0) // A[k1][k2] explicitely stored: retrieve it.
	return row_.columns_[key2_]; 
      else // A[k1][k2] not stored: return default value.
        return matrix().default_value();
      }
  private:
    SparseMatrix& matrix() const { return row_.matrix_; }
    Row&	  row_; // E.g. a[k1][k2] returns element refering to a[k1] row, key2=k2.
    Key2	  key2_;
  };

  SparseMatrix(const Value& default_value): default_(default_value) {}
  const Value&	default_value() const { return default_; }
  Row&		operator[](const Key1& k1) { 
		  // Return reference to row from rows_ (possibly after inserting it).
    		  return (*((rows_.insert(make_pair(k1,Row(*this)))).first)).second;
		  }
private:
  Value		default_;
  map<Key1,Row> rows_;
};
#endif
